<?php
namespace Teknisa\FIS\Modules\Acessorias\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fis06200DimeScFactory
{
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Executa a procedure para a exportação de DIME-SC [ARQUIVO_DIMESC]
     */
    public function callDimeScProcedure($params) {
        $procedure = new StoredProcedure($this->connection, 'ARQUIVO_DIMESC');
        
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $params["CDEMPRESA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDIMPOSTO', Param::PARAM_INPUT, $params["CDIMPOSTO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDINSCESTA', Param::PARAM_INPUT, $params["CDINSCESTA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $params["DTINICIAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $params["DTFINAL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DECLARACAO', Param::PARAM_INPUT, $params["DECLARACAO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_REGIME', Param::PARAM_INPUT, $params["REGIME"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSOLIDADA', Param::PARAM_INPUT, $params["CONSOLIDADA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CENTRALIZADA', Param::PARAM_INPUT, $params["CENTRALIZADA"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MOVIMENTO', Param::PARAM_INPUT, $params["MOVIMENTO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TRIBUTARIO', Param::PARAM_INPUT, $params["TRIBUTARIO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ESCCONTABIL', Param::PARAM_INPUT, $params["ESCCONTABIL"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CLASVENC', Param::PARAM_INPUT, $params["CLASVENC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NMTRAB', Param::PARAM_INPUT, $params["NMTRAB"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INCFISC', Param::PARAM_INPUT, $params["INCFISC"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INVENTARIO', Param::PARAM_INPUT, $params["INVENTARIO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQARQMAG', Param::PARAM_INPUT, $params["NRSEQARQMAG"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_GERACONTAB', Param::PARAM_INPUT, $params["GERACONTAB"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ATIVO', Param::PARAM_INPUT, $params["ATIVO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PASSIVO', Param::PARAM_INPUT, $params["PASSIVO"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DRE', Param::PARAM_INPUT, $params["DRE"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DESPESAS', Param::PARAM_INPUT, $params["DESPESAS"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREFCTB', Param::PARAM_INPUT, $params["DTREFCTB"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TPATIVIDADE', Param::PARAM_INPUT, $params["TPATIVIDADE"], Param::PARAM_TYPE_STR));
     
        $procedure->execute();
    }
    
    /*
     * Função para excluir os dados gerados na procedure de DIME-SC.
     */
    public function deleteDimeScData($params) {
        // Query a ser executada.
        $sql = "DELETE FROM ARQMAGNETICO WHERE NRSEQARQMAG = :NRSEQARQMAG AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);
    }

    /*
     * Função para retornar a sigla do estado relacionado a uma inscrição estadual.
     */
    public function getEstadoInscricaoEstadual($params) {
        // Query a ser executada.
        $sql = "SELECT SGESTADO FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA AND CDINSCESTA = :CDINSCESTA AND NRORG = :NRORG";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetch(\PDO::FETCH_ASSOC);
        
    }

    /*
     * Função para retornar os dados gerados na procedure de DIME-SC.
     */
    public function getDimeScData($params) {
        
        // Query a ser executada.
        $sql = "SELECT NRSEQARQMAG, NRORDREGARQ, DSREGARQMAG as DSREGARQMAG 
                FROM ARQMAGNETICO 
                WHERE NRORG = :NRORG 
                    AND NRSEQARQMAG = :NRSEQARQMAG 
                ORDER BY NRORDREGARQ, SUBSTR(DSREGARQMAG,1,17)";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetchAll();
    }    
    
    public function findAgendamento() {
        // Query a ser executada.
        $sql = "SELECT 
                    A.NRORG,
                    A.NRAGENDAMENTO,
                    A.CDTELA,
                    A.CDEMPRESA,
                    E.NMEMPRESA,
                    A.CDINSCESTA,
                    A.DTAGENDAMENTO,
                    A.DTINICIAL,
                    A.DTFINAL,
                    A.CDCAMPO1 AS CDIMPOSTONFE,
                    A.CDCAMPO2 AS GERAREGISTRO,
                    A.CDCAMPO3 AS CDATIVO,
                    A.CDCAMPO4 AS CDPASSIVO,
                    A.CDCAMPO5 AS CDDRE,
                    A.CDCAMPO6 AS CDDESPESAS,
                    DECODE(A.IDPERIAGEN, '0', A.DTINICIAL,
                                   '1', TO_CHAR(SYSDATE, 'DD/MM/YYYY'),
                                   '2', TO_CHAR(ADD_MONTHS(SYSDATE,(-1)), 'DD/MM/YYYY')) AS DTREFINI,
                    A.DTCAMPO2 AS DTREFCTB,
                    I.SGIMPOSTONFE,
                    A.IDPERIAGEN,
                    A.IDSTATUS
                    FROM AGENDOBRIGACESS A,
                    EMPRESA E,
                    IMPOSTONFE I
                    WHERE CDTELA  = 'FIS06200'
                    AND A.CDEMPRESA = E.CDEMPRESA
                    AND A.CDCAMPO1 = I.CDIMPOSTONFE";
                    

        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute();
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function insertAgendamento($row) {
        $sql = "INSERT INTO AGENDOBRIGACESS
            ( 
                NRORG,
                NRAGENDAMENTO,
                CDTELA,
            	CDEMPRESA,
                CDINSCESTA,
            	DTAGENDAMENTO,
            	DTINICIAL,
            	DTFINAL,
                IDPERIAGEN,
            	
            	CDCAMPO1,
                CDCAMPO2,
                CDCAMPO3,
                CDCAMPO4,
                CDCAMPO5,
                CDCAMPO6,
                DTCAMPO1, 
                DTCAMPO2   ) 
             VALUES (
                :NRORG,
                :NRAGENDAMENTO,
                :CDTELA,
                :CDEMPRESA,
                :CDINSCESTA,
                :DTAGENDAMENTO,
                :DTINICIAL,
                :DTFINAL,
                :IDPERIAGEN,
                :CDCAMPO1,
                :CDCAMPO2,
                :CDCAMPO3,
                :CDCAMPO4,
                :CDCAMPO5,
                :CDCAMPO6,
                :DTCAMPO1, 
                :DTCAMPO2 )";
                
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            
            //Campos da tabela que serão salvos e os campos da declaração
            "DTINICIAL" => $row["DTINICIAL"],
            "DTFINAL"   => $row["DTFINAL"],
            "IDPERIAGEN"=> $row["IDPERIAGEN"],
            "CDCAMPO1"  => $row["CDIMPOSTONFE"],
            "CDCAMPO2"  => $row["GERAREGISTRO"],
            "CDCAMPO3"  => $row["CDATIVO"],
            "CDCAMPO4"  => $row["CDPASSIVO"],
            "CDCAMPO5"  => $row["CDDRE"],
            "CDCAMPO6"  => $row["CDDESPESAS"],
            "DTCAMPO1"  => $row["DTREFINI"],
            "DTCAMPO2"  => $row["DTREFCTB"]
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateAgendamento($row) {
        $sql = "UPDATE AGENDOBRIGACESS
                    SET CDEMPRESA = :CDEMPRESA,
                        CDINSCESTA = :CDINSCESTA,
                    	DTAGENDAMENTO = :DTAGENDAMENTO,
                    	DTINICIAL = :DTINICIAL,
                    	DTFINAL = :DTFINAL,
                    	IDPERIAGEN = :IDPERIAGEN,
                    	CDCAMPO1 = :CDCAMPO1,
                        CDCAMPO2 = :CDCAMPO2,
                        CDCAMPO3 = :CDCAMPO3,
                        CDCAMPO4 = :CDCAMPO4,
                        CDCAMPO5 = :CDCAMPO5,
                        CDCAMPO6 = :CDCAMPO6,
                        DTCAMPO1 = :DTCAMPO1,
                        DTCAMPO2 = :DTCAMPO2
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                    AND CDTELA = :CDTELA";
                    
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            
            //Campos da tabela que serão salvos e os campos da declaração
            "DTINICIAL" => $row["DTINICIAL"],
            "DTFINAL"   => $row["DTFINAL"],
            "IDPERIAGEN"=> $row["IDPERIAGEN"],
            "CDCAMPO1"  => $row["CDIMPOSTONFE"],
            "CDCAMPO2"  => $row["GERAREGISTRO"],
            "CDCAMPO3"  => $row["CDATIVO"],
            "CDCAMPO4"  => $row["CDPASSIVO"],
            "CDCAMPO5"  => $row["CDDRE"],
            "CDCAMPO6"  => $row["CDDESPESAS"],
            "DTCAMPO1"  => $row["DTREFINI"],
            "DTCAMPO2"  => $row["DTREFCTB"]
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateAgendamentoIdStatus($row, $idstatus) {
        $sql = "UPDATE AGENDOBRIGACESS
                    SET IDSTATUS = :IDSTATUS
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                    AND CDTELA = :CDTELA";
                    
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "IDSTATUS"      => $idstatus
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getInscEsta($row) {
        // Query a ser executada.
        $sql = "SELECT CDINSCESTA FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA";

        $params = array(
            'CDEMPRESA' => $row['CDEMPRESA']
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function getAgendamentoInsert($row) {
        // Query a ser executada.
        $sql = "SELECT * FROM AGENDOBRIGACESS 
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";

        $params = array(
            "NRAGENDAMENTO"=>$row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function getConfiguraSMTP($row)  {
        // Query a ser executada.
        $sql = "SELECT * FROM CONFIGURASMTP
                    WHERE NRORG = :NRORG";
        
        $params = array(
            "NRORG"=> $row["NRORG"]
        );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function getContatos($row)  {
        // Query a ser executada.
        $sql = "SELECT * FROM CONTATOAPURAC
                    WHERE NRORG = :NRORG
                    AND CDEMPRESA = :CDEMPRESA
                    AND IDEXPORTAUTO = 'S'";
        
        $params = array(
            "NRORG"     => $row["NRORG"],
            "CDEMPRESA" => $row["CDEMPRESA"]
        );
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function getAgendamentoArquivo($row) {
        $sql = "SELECT * FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function deleteAgendamentoArquivo($row) {
        $sql = "DELETE FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
  
    public function deleteAgendamento($row) {
        // Query a ser executada.
        $sql = "DELETE FROM AGENDOBRIGACESS 
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";

        $params = array(
            "NRAGENDAMENTO"=>$row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
    }
    
    public function findAgendamentoExec($row) {
        // Query a ser executada.
        $sql = "SELECT 
                    TO_CHAR(SYSDATE, 'DD/MM/YYYY HH24:MI') AS DTATUAL,
                    TO_CHAR(ADD_MONTHS(SYSDATE,(-1)), 'DD/MM/YYYY HH24:MI') AS LAST_MONTH,
                    A.NRAGENDAMENTO,
                    A.CDTELA,
                    A.CDEMPRESA,
                    A.CDINSCESTA,
                    A.DTAGENDAMENTO,
                    A.DTINICIAL,
                    A.DTFINAL,
                    A.CDCAMPO1 AS CDIMPOSTONFE,
                    A.CDCAMPO2 AS GERAREGISTRO,
                    A.CDCAMPO3 AS CDATIVO,
                    A.CDCAMPO4 AS CDPASSIVO,
                    A.CDCAMPO5 AS CDDRE,
                    A.CDCAMPO6 AS CDDESPESAS,
                    A.DTCAMPO1 AS DTREFINI,
                    A.DTCAMPO2 AS DTREFCTB,
                    A.IDPERIAGEN,
                    A.IDSTATUS
                    FROM AGENDOBRIGACESS A
                    WHERE NRAGENDAMENTO = :NRAGENDAMENTO";
        
        $params = array(
            "NRAGENDAMENTO"=> $row["NRAGENDAMENTO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetchAll();
    }
    
    public function findParamDeclAcess($row) {
        // Query a ser executada.
        $sql = "SELECT
                 CDCAMPO1    AS STCLASSE,
                 CDCAMPO2    AS STNUMTRAB,
                 CDCAMPO3    AS STTIPODECLARACAO,
                 CDCAMPO4    AS STTIPOREGIME,
                 CDCAMPO5    AS STCONSOLIDADA,
                 CDCAMPO6    AS STCENTRALIZADA,
                 CDCAMPO7    AS STMOVIMENTO,
                 CDCAMPO8    AS STTRIBUTARIO,
                 CDCAMPO9    AS STESCCONTABIL,
                 CDCAMPO10   AS STINCFISCAL,
                 CDCAMPO12   AS STGERINVENTARIO,
                 CDCAMPO13   AS STTPATIVIDADE
             FROM
                 PARAMDECLACES
             WHERE
                 CDTELA =:CDTELA
                 AND CDEMPRESA =:CDEMPRESA";
        
        $params = array(
            "CDTELA"    => $row["CDTELA"],
            "CDEMPRESA" => $row["CDEMPRESA"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetch();
    }
    
    public function getNmEmpresa($row) {
        // Query a ser executada.
        $sql = "SELECT NMEMPRESA FROM EMPRESA
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND NRORG = :NRORG";
        
        $params = array(
            "CDEMPRESA" => $row["CDEMPRESA"],
            "NRORG"     => $row["NRORG"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        //var_dump($std->fetchAll());
        return $std->fetch();
    }
    
    public function verAgendamentoArquivo($row) {
        $sql = "SELECT * FROM ARQUIOBRIGACESS
                WHERE NRAGENDAMENTO = :NRAGENDAMENTO
                AND NMARQUIVO = :NMARQUIVO";
        
        $params = array(
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "NMARQUIVO"     => $row["NMARQUIVO"]
        );
        
        // Prepara o statement da query para execução.
        $std = $this->connection->prepare($sql);
        
        // Executa o statement com os paràmetros passados.
        $std->execute($params);
        
        // Executa um fetch no primeiro dado, retornando-o.
        return $std->fetchAll();
    }
    
    public function insertAgendamentoArquivo($row) {
        $sql = "INSERT INTO ARQUIOBRIGACESS
            ( 
                NRORG,
                NRARQUIVO,
                NRAGENDAMENTO,
                CDTELA,
                CDEMPRESA,
                CDINSCESTA,
                DTAGENDAMENTO,
                DTEXECAGEND,
                NMARQUIVO,
                NMPATHARQ
            )
             VALUES (
                :NRORG,
                :NRARQUIVO,
                :NRAGENDAMENTO,
                :CDTELA,
                :CDEMPRESA,
                :CDINSCESTA,
                :DTAGENDAMENTO,
                :DTEXECAGEND,
                :NMARQUIVO,
                :NMPATHARQ  )";
        
        $params = array(
            "NRORG"         => $row["NRORG"],
            "NRARQUIVO"     => $row["NRARQUIVO"],
            "NRAGENDAMENTO" => $row["NRAGENDAMENTO"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCESTA"    => $row["CDINSCESTA"],
            "DTAGENDAMENTO" => $row["DTAGENDAMENTO"],
            "DTEXECAGEND"   => $row["DTEXECAGEND"],
            "NMARQUIVO"     => $row["NMARQUIVO"],
            "NMPATHARQ"     => $row["NMPATHARQ"]
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
}