<?php

namespace Teknisa\FIS\Modules\Apuracoes\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Apuracoes\Factories\Fis41000ApuracaoAutomaticaFactory;
use Teknisa\FIS\Modules\Apuracoes\Services\Fis00100ApuracaoDeImpostoService;
use Teknisa\FIS\Modules\Apuracoes\Factories\Fis00100ApuracaoDeImpostoFactory;
use Teknisa\FIS\Modules\Acessorias\Services\Agendamento;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Masks\Masks;
use \Teknisa\Libs\Util\Libraries\PHPMailer;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Report\ReportService;
use Teknisa\Model\Fiscal\Entities\Mapaapurac;
use Teknisa\Model\Fiscal\Entities\Mapaapuracsn;
use DateTime;
use Zeedhi\Framework\Remote\FileUploader;
//use Zeedhi\FileServerAPI\API;

class Fis41000ApuracaoAutomaticaService {

    const DOCUMENTOS_FORA_PERIODO        = "NF_F_PER";
    const FAIXA_NAO_CADASTRADA           = "Faixa_NCad";
    const TABELA_SIMPLES_NAO_CADASTRADA  = "TabelaSimples_NCad";
    const IMPOSTOS_CONTRIBUICOES         = array("50","60");
    const IMPOSTOS_INSS                  = "42";
    const IMPOSTOS_IRPJCSLL              = array("25","70");
    const IMPOSTO_ICMSST                 = "80";

    protected $dataSourceManager;
    protected $entityManager;
    protected $environment;
    protected $factoryApuracaoAutomatica;
    protected $masks;
    protected $newCode;
    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $crudService;
    
    public $mensagemBody = '';
    public $mensagemBodySucess = '';
    public $mensagemError = '';
    public $mensagemErrorBody = '';
    public $geraArquivo = '';
    
    public function __construct($crudService, $dataSourceManager, \Doctrine\ORM\EntityManager $entityManager, Environment $environment, NewCode $newCode ,Fis41000ApuracaoAutomaticaFactory $factoryApuracaoAutomatica, 
                                Fis00100ApuracaoDeImpostoService $serviceApuracaoImposto, Fis00100ApuracaoDeImpostoFactory $factoryApuracaoDeImposto, Agendamento $agendamento, Masks $masks,
                                ReportService $reportService, $fileServerUrl, $fileUploader) {
        $this->crudService                  = $crudService;
        $this->dataSourceManager            = $dataSourceManager;
        $this->entityManager                = $entityManager;
        $this->environment                  = $environment;
        $this->factoryApuracaoAutomatica    = $factoryApuracaoAutomatica;
        $this->factoryApuracaoDeImposto     = $factoryApuracaoDeImposto;
        $this->serviceApuracaoImposto       = $serviceApuracaoImposto;
        $this->agendamento                  = $agendamento;
        $this->masks                        = $masks;
        $this->reportService                = $reportService;
        $this->newCode                      = $newCode;
        $this->nrorg                        = $environment->getNrorgTrab();
        $this->operador                     = $this->environment->getCdOperador();
        $this->currentTime                  = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function salvarApuracAutomatica ($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            if(is_array($row['CDINSCESTA'])){
                if($row['IDTPAPAUTO'] == 'L'){
                    $arrayInscEsta = $row['CDINSCESTA']; 
                    $cdinscesta = '_';
                    
                    foreach($arrayInscEsta as $value){
                        $cdinscesta = $cdinscesta.$value.'_';
                    }
                    
                    $row['CDINSCESTARRAY'] = $cdinscesta;
                    $row['CDINSCESTA'] = ""; 
                }
            } else {
                $row['CDINSCESTARRAY'] = $row['CDINSCESTA'];
                $row['CDINSCESTA'] = ""; 
            }
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $row['NRSEQUJOB'] = $this->newCode->getCodeByTableName('MAPAAPURACAUTO', $this->nrorg);
                
                $this->factoryApuracaoAutomatica->insertApuraAuto($row); 
            } else {
                $row['IDSTATUS'] = 'P';
                //APAGAR ARQUIVOS EXISTENTES
                $verifArq = $this->factoryApuracaoAutomatica->getArquivosUpdate($row);
                
                foreach($verifArq as $arquivo){
                    $this->deleteArquivos($arquivo);
                }
                
                $this->factoryApuracaoAutomatica->updateApuraAuto($row); 
            }
            
           $apuracaoAuto = $this->factoryApuracaoAutomatica->getApuraAutoInsert($row);
            
            if($apuracaoAuto){
                
                $apuracaoAuto['NRORG'] = $row['NRORG'];
                $getConfigSMTP = $this->factoryApuracaoAutomatica->getConfiguraSMTP($apuracaoAuto)[0];
                
                if(isset($row['__is_new']) && $row['__is_new'] == true){
                    $nomeJob = 'APURACAO_AUTOMATICA_'.$row['NRSEQUJOB'];
                } else {
                    $nomeJob = 'APURACAO_AUTOMATICA_'.str_pad($row['NRSEQUJOB'], 10, "0", STR_PAD_LEFT);
                }
                
                //CRIA JOB VERIFICAÇÃO
                if(!empty($row['DTVDPERIODO'])){
                    $dataContabAuto = $row['DTVDPERIODO'];
                    $sequencialJob = $row['NRSEQUJOB'];
                    $nrorg = $row['NRORG'];
                    $descricao = 'Verificação da Apuração Automática '.$row['NRSEQUJOB'].' na data de '.$row['DTVDPERIODO'];
                    //Conferir caminho do diretorio do xampp
                    $arqCallService = $getConfigSMTP['CALLSERVPATH'];//'C:\Teknisa\CallService.php';
                    //$dirphp = str_replace('/','\',$DIRPHP);
                    $dirphp = $getConfigSMTP['DIRPHP'];//'C:/xampp/php';
                    $jenkinsPath = $getConfigSMTP['JENKINSPATH'];//'http://192.168.120.132:8080/';
                    $execPath = '/execVerificaApuracaoAuto';
                    $periodicidade = '0';//$row['IDPERIAGEN'];
                    $jenkinsCreateItem = $getConfigSMTP['JENKCREATITEM'];
                    $this->agendamento->criaJob('VERIFICACAO_'.$nomeJob, $dataContabAuto, $sequencialJob, $nrorg, $descricao, $arqCallService, $dirphp, $jenkinsPath, $execPath, $periodicidade, $jenkinsCreateItem);
                }
                
                //CRIA JOB APURAÇÃO
                $dataContabAuto = $row['DTJOBAPURAC'];
                $sequencialJob = $row['NRSEQUJOB'];
                $nrorg = $row['NRORG'];
                $descricao = 'Apuração Automática '.$row['NRSEQUJOB'].' na data de '.$row['DTJOBAPURAC'];
                //Conferir caminho do diretorio do xampp
                $arqCallService = $getConfigSMTP['CALLSERVPATH'];//'C:\Teknisa\CallService.php';
                //$dirphp = str_replace('/','\',$DIRPHP);
                $dirphp = $getConfigSMTP['DIRPHP'];//'C:/xampp/php';
                $jenkinsPath = $getConfigSMTP['JENKINSPATH'];//'http://192.168.120.132:8080/';
                $execPath = '/execApuracaoAuto';
                $periodicidade = '0';//$row['IDPERIAGEN'];
                $jenkinsCreateItem = $getConfigSMTP['JENKCREATITEM'];
                $this->agendamento->criaJob($nomeJob, $dataContabAuto, $sequencialJob, $nrorg, $descricao, $arqCallService, $dirphp, $jenkinsPath, $execPath, $periodicidade, $jenkinsCreateItem);
                   
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function execApuracaoAuto($row, $verificacao = false) {
        try {
            //Ao aproveitar a função ja existente de agendamento, o parametro é passado como NRAGENDAMENTO do Jenkins
            $row['NRSEQUJOB'] = $row['NRAGENDAMENTO'];
            
            $apuracao = $this->factoryApuracaoAutomatica->selectApuraAutoExec($row)[0];
            
            $apuracao['NRSEQUJOB'] = $row['NRSEQUJOB'];
            
            if($apuracao){
                if($apuracao['IDSTATUS'] != 'R'){
                    
                    $this->mensagemBody = '';
                        
                    $arrayInscEsta = array();
                    if($apuracao['IDTPAPAUTO'] == 'T'){
                        foreach($this->factoryApuracaoAutomatica->getInscrestadEmpresa($apuracao) as $result){
                            array_push($arrayInscEsta, $result['CDINSCESTA']);
                        }
                    } else if($apuracao['IDTPAPAUTO'] == 'A'){
                        foreach($this->factoryApuracaoAutomatica->getAgrupamento($apuracao['CDGRUPINSCESTA']) as $result){
                            array_push($arrayInscEsta, $result['CDINSCESTA']);
                        }
                    } else if($apuracao['IDTPAPAUTO'] == 'L'){
                        $arrayInscEsta = explode('_',$apuracao['CDINSCESTARRAY']);
                    }
                    
                    $getConfigSMTP = $this->factoryApuracaoAutomatica->getConfiguraSMTP($apuracao)[0];
                    $getContatos = $this->factoryApuracaoAutomatica->getContatos($apuracao);
                    
                    $mail = new PHPMailer;
                    $mail->CharSet = 'UTF-8';
                    $mail->isSMTP();                        // Set mailer to use SMTP
                    $mail->isHTML();
                    $mail->Host = $getConfigSMTP['SERVIDORSMTP'];       // Specify main and backup SMTP servers
                    $mail->SMTPAuth = true;                 // Enable SMTP authentication
                    $mail->Username = $getConfigSMTP['LOGINSMTP'];    // SMTP username
                    $mail->Password = $getConfigSMTP['SENHASMTP'];    // SMTP password
                    $mail->From = $getConfigSMTP['LOGINSMTP'];
                    $mail->FromName = 'ERP Pebbian';
                    $mail->Subject = 'Apuração Automática de '.$apuracao['DTJOBAPURAC'];
                    
                    foreach($getContatos as $contato){
                        $mail->addAddress($contato['EMAIL']);
                    };
                    
                    $apuracao["NMEMPRESA"] = $this->factoryApuracaoAutomatica->getNmEmpresa($apuracao)["NMEMPRESA"];
                            
                    $this->mensagemBody = 'Caro cliente, '.'<br />'.'<br />';
                    if($verificacao == false){
                        $this->mensagemBodySucess = 'Apuração Fiscal realizada com Sucesso!'
                        .'<br />'.'Empresa: '.$apuracao['CDEMPRESA'].' - '.$apuracao['NMEMPRESA'];
                        $this->mensagemErrorBody = 'Erro ao apurar:'.'<br />'.'<br />'
                        .'<br />'.'Empresa: '.$apuracao['CDEMPRESA'].' - '.$apuracao['NMEMPRESA'];
                    } else {
                        $this->mensagemBodySucess = 'Verificação da Apuração Fiscal realizada com Sucesso!'
                        .'<br />'.'Empresa: '.$apuracao['CDEMPRESA'].' - '.$apuracao['NMEMPRESA'];
                        $this->mensagemErrorBody = 'Erro ao verificar:'.'<br />'.'<br />'
                        .'<br />'.'Empresa: '.$apuracao['CDEMPRESA'].' - '.$apuracao['NMEMPRESA'];
                    }
                    $contErro = 0;
                    $contSucess = 0;
                        
                    
                    foreach($arrayInscEsta as $inscEsta){
                        if(($inscEsta) && $inscEsta != ""){
                            
                            $this->mensagemError = '';
                
                            $apuracao['CDINSCESTA'] = $inscEsta;
                            
                            $existeMapa = $this->factoryApuracaoAutomatica->verificaMapaApura($apuracao);
                            //var_dump('Inscricao: '.$apuracao['CDINSCESTA']);
                            
                            if(empty($existeMapa)){
                                //NÃO EXISTE MAPA NO PERIODO DESCRITO. QUAL O ULTIMO MAPA'?'
                                //var_dump('Nao Existe Mapa para Inscricao acima.');
                                
                                $apuracao['IDTIPOAPURA'] = $this->factoryApuracaoAutomatica->buscaTipoApura($apuracao["CDIMPOSTO"])['IDORGARRECADNFE'];
                                
                                $ultimoMapa = $this->factoryApuracaoAutomatica->buscaUltimoMapa($apuracao);
                                
                                if(empty($ultimoMapa)){
                                    //NÃO EXISTE MAPA PARA A EMPRESA IMPOSTO E INSCRIÇÃO INDICADOS. CRIA UM NOVO MAPA E APURA O MESMO
                                    
                                    //var_dump('Criando um novo Mapa.');
                                
                                    $apuracao['NRSEQUMAPAPU'] = $this->newCode->getCodeByTableName('MAPAAPURACAO', $this->nrorg);
                                    
                                    $this->factoryApuracaoAutomatica->insertMapaApurac($apuracao);
                                    
                                    $novoMapa = $this->factoryApuracaoAutomatica->getMapaApurac($apuracao['NRSEQUMAPAPU']);
                                    
                                    $novoMapa['IDTIPOAPURA'] = $this->factoryApuracaoAutomatica->buscaTipoApura($apuracao["CDIMPOSTO"])['IDORGARRECADNFE'];
                                    
                                    $avisos = $this->apurarMapa($novoMapa, $bloqueio = ($apuracao['IDTPAPURACAO'] == 'D') ? true : false, $verificacao);
                                    //ATUALIZAR MAPAAPURACAUTO PARA REALIZADO
                                    
                                } else {
                                    //NÃO EXISTE MAPA NO PERIODO DESCRITO, O ULTIMO MAPA CADASTRADO FOI O RETORNO DO SELECT
                                    //COMPARA DTFIM ULTIMO MAPA COM DTINI DESCRITA. SE FOR COMPATIVEL CRIA MAPA E APURA, SE NAO RE
                                    //RETORNA ERRO
                                    
                                    if(DateUtil::getQtdeDiasEntre(substr($ultimoMapa['DTFIMPERIODO'], 0, 10), substr($apuracao['DTINIPERIODO'], 0, 10)) == 1){
                                        //var_dump("Ultimo mapa e' subsequente ao cadastrado.");
                                        
                                        $apuracao['NRSEQUMAPAPU'] = $this->newCode->getCodeByTableName('MAPAAPURACAO', $this->nrorg);
                                    
                                        $this->factoryApuracaoAutomatica->insertMapaApurac($apuracao);
                                        
                                        $novoMapa = $this->factoryApuracaoAutomatica->getMapaApurac($apuracao['NRSEQUMAPAPU']);
                                        
                                        $novoMapa['IDTIPOAPURA'] = $this->factoryApuracaoAutomatica->buscaTipoApura($apuracao["CDIMPOSTO"])['IDORGARRECADNFE'];
                                        
                                        $avisos = $this->apurarMapa($novoMapa, $bloqueio = ($apuracao['IDTPAPURACAO'] == 'D') ? true : false, $verificacao);
                                        
                                        //ATUALIZAR MAPAAPURACAUTO PARA REALIZADO
                                        
                                    } else {
                                        //var_dump("Erro ao apurar o mapa. Nao foi possivel gerar o mapa pois a data da apuração nao e' subsequente a data do ultimo mapa cadastrado.");
                                        $this->mensagemError = '<br />'.'Erro ao apurar o mapa. Não foi possível gerar o mapa pois a data da apuração 
                                        nao é subsequente a data do ultimo mapa cadastrado.';
                                    }
                                }
                            } else{
                                //EXISTE MAPA NO PERIODO DESCRITO -> APURA MAPA
                                
                                //var_dump('Mapa ja existe.');
                                
                                if($existeMapa['IDBLOQMAPAPU'] != 'S'){        
        
                                    $existeMapa['IDTIPOAPURA'] = $this->factoryApuracaoAutomatica->buscaTipoApura($apuracao["CDIMPOSTO"])['IDORGARRECADNFE'];
                                    
                                    $apuracao['NRSEQUMAPAPU'] = $existeMapa['NRSEQUMAPAPU'];
                                    
                                    $avisos = $this->apurarMapa($existeMapa, ($apuracao['IDTPAPURACAO'] == 'D') ? true : false, $verificacao);
                                    
                                    //ATUALIZAR MAPAAPURACAUTO PARA REALIZADO
                                } else if($existeMapa['IDBLOQMAPAPU'] == 'S'){
                                    //var_dump('Nao foi possivel apurar o mapa '.$existeMapa['NRSEQUMAPAPU'].' - '.$existeMapa['NMMAPAAPUR'].'. Mapa esta Bloqueado.');
                                    $this->mensagemError = '<br />'.'Não foi possivel Apurar o mapa '.$existeMapa['NRSEQUMAPAPU'].' - '.$existeMapa['NMMAPAAPUR'].'. Mapa está Bloqueado.';
                                }
                            }
                            
                            if(($this->mensagemError == '') && $verificacao == false){
                                
                                $this->mensagemBodySucess = $this->mensagemBodySucess
                                .'<br />'.'<br />'.'Inscrição Estadual: '.$apuracao['CDINSCESTA'];
                                ++$contSucess;
                                //GERAR RELATORIO AQUI
                                
                                $arrayReportParam = $this->getReport($apuracao);
                                
                                foreach($arrayReportParam as $reportParam){
                                    
                                    
                                    $reportStrategy = $reportParam['strategy'];
                                    $reportName = $reportParam['name'];
                                    $parametersRow = $reportParam['parameterMapping'];
                                    $extensao = $reportParam['extension'];
                                    
                                    try{
                                        $PDFreport = $this->reportService->createRemoteReport($reportStrategy, $reportName, $parametersRow);
                                    } catch (\Exception $e) {
                                        //var_dump('Erro ao gerar arquivo '.$reportParam['name'].'.');
                                        $this->mensagemBodySucess = $this->mensagemBodySucess
                                        .'<br />'.'Erro ao gerar arquivo '.$reportParam['nmarquivo']
                                        .'<br />'.'Erro: '. (($e->getMessage() == 'No data to be listed') ? 'Não Existem dados para serem Listados' : $e->getMessage());
                                        continue;
                                    }
                                    
                                    $apuracao['NMARQUIVO'] = 'APURACAO_AUTOMATICA_'.$row['NRSEQUJOB'].'_'.$apuracao['CDEMPRESA'].'_'.$apuracao['CDINSCESTA'].'_'.$reportParam['name'];
                                    
                                    $apuracao['NMPATHARQ'] = $this->salvarRelatorio($apuracao['NMARQUIVO'], $PDFreport, $getConfigSMTP['CALLSERVPATH'], $extensao);
                                    
                                    $verifArq = $this->factoryApuracaoAutomatica->verificaArquivo($apuracao);
                                        
                                    if(empty($verifArq)){
                                        //var_dump('Gerando arquivo '.$reportParam['name'].'.');
                                        $apuracao['NMARQUIVO'] = $reportParam['nmarquivo'].' - Apuração Automática '.$row['NRSEQUJOB'].' - Empresa '.$apuracao['CDEMPRESA'].' - Inscrição '.$apuracao['CDINSCESTA'];
                                        $apuracao["NRARQUIVO"] = $this->newCode->getCodeByTableName('NRARQUIVO', $apuracao["NRORG"], 1, 10);
                                        $apuracao["DTEXECAGEND"] = date('d/m/Y H:i');
                                        
                                        $this->factoryApuracaoAutomatica->insertArquivo($apuracao);
                                        
                                        $this->mensagemBodySucess = $this->mensagemBodySucess
                                        .'<br />'.'Arquivo '.$reportParam['nmarquivo'].' gerado com sucesso.';
                                    }
                                    
                                }
                            }   
                            else if(($this->mensagemError == '') && $verificacao == true){
                                
                                $this->mensagemBodySucess = $this->mensagemBodySucess
                                .'<br />'.'Inscrição Estadual: '.$apuracao['CDINSCESTA'];
                                ++$contSucess;
                                
                            } 
                            else if(isset($avisos)) {
                                if((!empty($avisos)) || ($avisos != false)) {
                                    $this->mensagemErrorBody = $this->mensagemErrorBody
                                    .'<br />'.'Inscrição Estadual: '.$apuracao['CDINSCESTA'];
                                    foreach($avisos as $aviso) {
                                        $this->mensagemErrorBody = $this->mensagemErrorBody
                                        .$aviso.'<br />'.'<br />';
                                    }
                                    ++$contErro;
                                }
                            } 
                            else {
                                $this->mensagemErrorBody = $this->mensagemErrorBody
                                .'<br />'.'Inscrição Estadual: '.$apuracao['CDINSCESTA']
                                .$this->mensagemError.'<br />'.'<br />';
                                ++$contErro;
                            }
                        }
                    }
                    
                    $this->factoryApuracaoAutomatica->updateStatusMapaApuracAuto($apuracao);
                    
                    //INSERIR AVISOS NOS ERROS.
                    if(($contErro > 0) && ($contSucess > 0)){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'.'<br />'
                        .'----------------------------------------------------------------------------------------'.'<br />'
                        .$this->mensagemErrorBody.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    } 
                    else if($contErro > 0){
                        $mail->Body = $this->mensagemBody.'<br />'
                        .$this->mensagemErrorBody.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    else if($contSucess > 0){
                        $mail->Body = $this->mensagemBody.'<br />'.$this->mensagemBodySucess.'<br />'
                        .'<br />'.'Atenciosamente,'.'<br />'.'<br />'.'Equipe ERP Pebbian.';
                        $mail->send();
                    }
                    
                }
            }    
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function apurarMapa($row, $bloquearMapa = false, $verificacao) {
        $avisos = $this->validaApuracao($row, $bloquearMapa);
        if(($avisos != 'Exception') && ($verificacao == false)){    
            try {
                //var_dump('Apurando.');
                                
                $aviso = $this->callProcedureApurarMapa($row, $bloquearMapa);
                // Verifica se algum aviso foi retornado.
                if($aviso == 'Exception'){    
                    $this->entityManager->rollback();
                    return false;
                    
                } else {
        
                    if (!empty($aviso)) {
                        $avisos[] = $aviso;
                    }
        
                    $params = array(
                        'nrorg' => $this->nrorg,
                        'nrsequmapapu' => $row['NRSEQUMAPAPU']
                    );
                    // Verifica se o mapa selecionado é de Simples Nacional.
                    if ($this->isSimplesNacional($row)) {
                        $currentTime = $this->environment->getCurrentTime();
                        $mapaapurac = $this->entityManager->getRepository(Mapaapuracsn::class)->findOneBy($params);
                        // Seta a data de realização da apuração para o Simples.
                        $mapaapurac->setDtrealapura($currentTime);
                    }
                    // Se não for, busca da tabela geral.
                    else {
                        $mapaapurac = $this->entityManager->getRepository(Mapaapurac::class)->findOneBy($params);
                    }
                    if($bloquearMapa == false){    
                        /// Seta o mapa como apurado (A).
                        $mapaapurac->setIdbloqmapapu('A');
                    } 
                    else {
                        /// Seta o mapa como Bloqueado (S).
                        $mapaapurac->setIdbloqmapapu('S');
                    }
                    $this->entityManager->persist($mapaapurac);
                    $this->entityManager->flush();
                }
                
            } catch (\Exception $e) {
                $this->entityManager->rollback();
                throw $e;
            }
    
            return $avisos;
        } else {
            return false;
        }
    }
    
    private function validaApuracao($row, $bloquearMapa) {
        $mapaAnterior = $this->buscaMapaAnterior($row);
        // Valida se o mapa anterior foi previamente apurado ou bloqueado, caso
        // a apuração seja para bloquear o mapa.
        if ($mapaAnterior && $mapaAnterior['IDBLOQMAPAPU'] != 'S' && ($mapaAnterior['IDBLOQMAPAPU'] != 'A' || $bloquearMapa)) {
            //throw new \Exception("Mapa anterior deve estar " . ($bloquearMapa ? "bloqueado." : "apurado."));
            $this->mensagemError = $this->mensagemError.'<br />'."Mapa anterior deve estar " . ($bloquearMapa ? "bloqueado." : "apurado.");
            return 'Exception';
        }

        $params = array(
            "NRORG" => $this->nrorg
        );
        $dataInicialNota = $this->factoryApuracaoDeImposto->buscaDataParametroFiscal($params);
        // A data deve estar parametrizada.
        if (!$dataInicialNota) {
            //throw new \Exception("Deve ser informada na tela Parâmetros de Fiscal a data inicial para buscar notas fiscais fora do período de apuração.");
            $this->mensagemError = $this->mensagemError.'<br />'.'Deve ser informada na tela Parâmetros de Fiscal a data inicial para buscar notas fiscais fora do período de apuração.';
            return 'Exception';
        }
        $dataInicialNota = $this->formataData($dataInicialNota);
        $dataMapa = $this->formataData($row['DTINIPERIODO']);
        // A data parametrizada deve vir antes da data do mapa.
        if (DateUtil::comparaDatas($dataInicialNota, $dataMapa, DateUtil::DEPOIS)) {
            //throw new \Exception("Data inicial para verificação de notas fiscais fora do período deve ser menor que a data inicial do mapa de apuração.");
            $this->mensagemError = $this->mensagemError.'<br />'.'Data inicial para verificação de notas fiscais fora do período deve ser menor que a data inicial do mapa de apuração.';
            return 'Exception';
        }

        if ($this->isContribuicoes($row)) {
            return $this->validaApuracaoContribuicoes($row, $bloquearMapa);
        }
        // Não executa validações extras para o Simples Nacional.
        else if ($this->isSimplesNacional($row)) {
            // Retorna um array vazio.
            return array();
        }
        
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $row["CDIMPOSTO"]
        );
        $formaApuracao = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDFORMAPURNFE"];

        // Inicializa a variável de avisos que não bloqueiem a apuração.
        $avisos = array();

        // Verifica se o mapa é municipal para realização de validações de
        // incidência que não são consideradas em apurações municipais.
        if ($row["IDTIPOAPURA"] != 'M') {
            $params = array(
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDIMPOSTO"     => $row["CDIMPOSTO"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "NRORG"         => $this->nrorg,
            );
            // Verifica se há alguma nota fiscal pendente de aprovação ou sem
            // incidência de imposto.
            $notaPendente = $this->factoryApuracaoDeImposto->getNotaPendente($params);
            if ($notaPendente) {
                if ($bloquearMapa) {
                    //throw new \Exception("Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.");
                    $this->mensagemError = $this->mensagemError.'<br />'.'Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.';
                    return 'Exception';
                }
                $avisos[] = 'Existem notas fiscais pendentes de aprovação/emissão ou sem incidência do imposto, favor verificar o relatório "Nota Fiscal Pendente" na tela Conferência Fiscal.';
            }
            $params = array(
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDINSCESTA"    => $row["CDINSCESTA"],
                "NRORG"         => $this->nrorg,
            );
            // Realiza verificação de venda pendente de aprovação ou sem incidência
            // de imposto.
            $vendaPendente = $this->factoryApuracaoDeImposto->getVendaPendente($params);
            if ($vendaPendente) {
                if ($bloquearMapa) {
                    //throw new \Exception("Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.");
                    $this->mensagemError = $this->mensagemError.'<br />'.'Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.';
                    return 'Exception';
                }
                $avisos[] = 'Existem vendas pendentes de aprovação/emissão ou sem incidência do imposto.';
            }
        }
        return $avisos;
    }
    
    private function callProcedureApurarMapa($row,  $bloquearMapa) {
        $params = array(
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "DTINIPERIODO"  => $row["DTINIPERIODO"],
            "DTFIMPERIODO"  => $row["DTFIMPERIODO"],
            "NRORG"         => $this->nrorg
        );
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $codigoRetorno = $this->factoryApuracaoDeImposto->apuraMapaSimples($params);
            if ($codigoRetorno == self::FAIXA_NAO_CADASTRADA) {
                //throw new \Exception("Não há Faixa de Valores correspondente ao Valor do Faturamento dos últimos 12 meses (R$).");
                $this->mensagemError = $this->mensagemError.'<br />'.'Não há Faixa de Valores correspondente ao Valor do Faturamento dos últimos 12 meses (R$).';
                return 'Exception';
            }
            else if ($codigoRetorno == self::DOCUMENTOS_FORA_PERIODO) {
                //throw new \Exception("Na apuração existem documentos fora do período.");
                $this->mensagemError = $this->mensagemError.'<br />'.'Na apuração existem documentos fora do período.';
                return 'Exception';
            }
            else if ($codigoRetorno == self::TABELA_SIMPLES_NAO_CADASTRADA) {
                //throw new \Exception("Não existe Tabela do Simples Nacional relacionada à Empresa.");
                $this->mensagemError = $this->mensagemError.'<br />'.'Não existe Tabela do Simples Nacional relacionada à Empresa.';
                return 'Exception';
            }
        }
        // Verifica se o mapa é de Contribuições.
        else if ($this->isContribuicoes($row)) {
            $params = array(
                "CDIMPOSTONFE" => $row["CDIMPOSTO"],
                "NRORG" => $this->nrorg
            );
            // Retorna o parâmetro de imposto recuperável para o imposto do
            // mapa.
            $idimporecupnfe = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDIMPORECUPNFE"];
            $params = array(
                "DTINICIAL"         => $row["DTINIPERIODO"],
                "DTFINAL"           => $row["DTFIMPERIODO"],
                "IDIMPORECUPNFE"    => $idimporecupnfe,
                "CDIMPOSTONFE"      => $row["CDIMPOSTO"],
                "NRSEQUMAPAPU"      => $row["NRSEQUMAPAPU"],
                "CDEMPRESA"         => $row["CDEMPRESA"],
                "P_BOBLOQUEIA"      => $bloquearMapa
            );
            $this->factoryApuracaoDeImposto->apuraMapaContribuicoes($params);
        }
        else if($this->isInss($row)){
            $this->factoryApuracaoDeImposto->calculaInss(
                array(
                    
                    "P_VRBCFOLHA"      => $row["VRTTBACALDEB"],
                    "P_VRAPURFOLHA"    => $row["VRTTCTBLDEB"],
                    "P_CDEMPRESA"      => $row["CDEMPRESA"],
                    "P_CDIMPOSTONFE"   => $row["CDIMPOSTO"],
                    "P_NRSEQUMAPAPU"   => $row['NRSEQUMAPAPU'],
                    "P_DTINICIAL"      => $row["DTINIPERIODO"],
                    "P_DTFINAL"        => $row["DTFIMPERIODO"],
                    "P_IDCUMUPISCOFIL" => null
                )
            );
            $this->entityManager->flush();
            
            $this->factoryApuracaoDeImposto->calculaTotalInss(array("P_NRSEQUMAPAPU"=> $row['NRSEQUMAPAPU']));
            $this->entityManager->flush();
        }
        else {
            $inscricao = $row["IDTIPOAPURA"] == 'E' ? $row["CDINSCESTA"] : ($row["IDTIPOAPURA"] == 'M' ? $row["CDINSCMUNI"] : null);
            $params = array(
                "CDIMPOSTO"     => $row["CDIMPOSTO"],
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDINSCRICAO"   => $inscricao,
                "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
                "DTINICIAL"     => $row["DTINIPERIODO"],
                "DTFINAL"       => $row["DTFIMPERIODO"],
                "BOBLOQ"        => $bloquearMapa ? 'S':'N'
            );

            $strRetorno = $this->factoryApuracaoDeImposto->apuraMapa($params);
            //Somente quando está apurando o ICMS ST pela primeira vez, gera as guias referentes
            if( $row['CDIMPOSTO'] === self::IMPOSTO_ICMSST ){
                if( empty( $this->factoryApuracaoDeImposto->verificaGuiaIcmsSt( array("P_NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]) ) ) )  
                    $this->factoryApuracaoDeImposto->gerarGuiaIcmsSt(array("NRSEQUMAPAPU"=>$row["NRSEQUMAPAPU"]));
            }
            // Verifica se foi retornado 'OK', indicando que não houve erros.
            if (substr($strRetorno, 0, 2) === 'OK') {
                // Se houver alguma nota pendente, haverá um 'N' após o 'OK',
                // indicando que a apuração ocorreu com sucesso, mas há notas
                // pendentes ou sem incidência para o mapa.
                if (substr($strRetorno, 2, 1) === 'S') {
                    return 'Existem notas fora do período na apuração.';
                }
            }
            // Se não houver o 'OK', retornará uma mensagem de erro.
            else {
                if ($bloquearMapa) {
                    //throw new \Exception($strRetorno);
                    $this->mensagemError = $this->mensagemError.'<br />'.$strRetorno;
                    return 'Exception';
                }
                return $strRetorno;
            }
        }
    }

    private function buscaMapaAnterior($row) {
        // Verifica se o mapa é de Simples Nacional.
        if ($this->isSimplesNacional($row)) {
            $params = array(
                "NRORG" => $this->nrorg,
                "CDEMPRESA" => $row['CDEMPRESA'],
                "DTINIPERIODO" => $row['DTINIPERIODO']
            );
            $mapaAnterior = $this->factoryApuracaoDeImposto->buscaMapaAnteriorSimples($params);
        }
        else {
            $params = array(
                "NRORG"         => $this->nrorg,
                "CDEMPRESA"     => $row['CDEMPRESA'],
                "CDIMPOSTO"     => $row['CDIMPOSTO'],
                "CDINSCESTA"    => $row['CDINSCESTA'],
                "CDINSCMUNI"    => $row['CDINSCMUNI'],
                "IDTIPOAPURA"   => $row['IDTIPOAPURA'],
                "DTINIPERIODO"  => $row['DTINIPERIODO']
            );
            $mapaAnterior = $this->factoryApuracaoDeImposto->buscaMapaAnterior($params);
        }
        return $mapaAnterior;
    }
    
    public function isSimplesNacional($row) {
        return $row["IDTIPOAPURA"] == 'S';
    }
    
    public  function isContribuicoes($row) {
        return in_array($row["CDIMPOSTO"], self::IMPOSTOS_CONTRIBUICOES);
    }
    
    public function isInss($row) {
        return $row["CDIMPOSTO"] == self::IMPOSTOS_INSS;
    }
    
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    private function validaApuracaoContribuicoes($row, $bloquearMapa) {
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $row["CDIMPOSTO"]
        );
        $formaApuracao = $this->factoryApuracaoDeImposto->getImpostoNfe($params)["IDFORMAPURNFE"];

        // Inicializa a variável de avisos que não bloqueiem a apuração.
        $avisos = array();

        $params = array(
            "NRSEQUMAPAPU"  => $row["NRSEQUMAPAPU"],
            "CDIMPOSTONFE"  => $row["CDIMPOSTO"],
            "DTFINFORPER"   => $row["DTINIPERIODO"],
            "IDFORMAPURNFE" => $formaApuracao,
            "NRORG"         => $this->nrorg,
        );
        $notaFiscal = $this->factoryApuracaoDeImposto->getNotaForaPeriodoContribuicoes($params);
        // Verifica se há alguma nota fiscal fora do período de apuração.
        if ($notaFiscal) {
            $errorMessage = "Existem notas fora do período na apuração.";
            $avisos[] = $errorMessage;
        }

        $params = array(
            "NRSEQUMAPAPU"      => $row["NRSEQUMAPAPU"],
            "DTFINFORPER"       => $row["DTINIPERIODO"],
            "IDFORMAPURNFE"     => $formaApuracao,
            "NRORG"             => $this->nrorg,
        );
        // Verifica se há alguma nota fiscal de compensação fora do período de
        // apuração.
        $notaCompensacao = $this->factoryApuracaoDeImposto->getNotaCompensacaoForaPeriodo($params);
        if ($notaCompensacao) {
            $errorMessage = "Existem notas de compensação fora do período na apuração.";
            $avisos[] = $errorMessage;
        }

        $params = array(
            "DTINICIAL"     => $row["DTINIPERIODO"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "DTFINAL"       => $row["DTFIMPERIODO"],
            "CDIMPOSTO"     => $row["CDIMPOSTO"],
            "IDFORMAPUR"    => $formaApuracao,
            "NRORG"         => $this->nrorg
        );
        // Verifica se há alguma nota fiscal pendente de aprovação ou sem
        // incidência de imposto.
        $notaPendente = $this->factoryApuracaoDeImposto->getNotaPendenteContribuicoes($params);
        if ($notaPendente) {
            if ($bloquearMapa) {
                //throw new \Exception("Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.");
                $this->mensagemError = $this->mensagemError.'<br />'.'Operação Bloqueada! Existem notas fiscais pendentes de aprovação/emissão ou sem incidência de imposto.';
                return 'Exception';
            }
            $avisos[] = 'Existem notas fiscais pendentes de aprovação/emissão ou sem incidência do imposto, favor verificar o relatório "Nota Fiscal Pendente" na tela Conferência Fiscal.';
        }

        $params = array(
            "DTINICIAL" => $row["DTINIPERIODO"],
            "DTFINAL"   => $row["DTFIMPERIODO"],
            "CDEMPRESA" => $row["CDEMPRESA"],
            "NRORG"     => $this->nrorg,
        );
        // Realiza verificação de venda pendente de aprovação ou sem incidência
        // de imposto.
        $vendaPendente = $this->factoryApuracaoDeImposto->getVendaPendenteContribuicoes($params);
        if ($vendaPendente) {
            if ($bloquearMapa) {
                //throw new \Exception("Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.");
                $this->mensagemError = $this->mensagemError.'<br />'.'Operação Bloqueada! Existem vendas pendentes de aprovação/emissão ou sem incidência de imposto.';
                return 'Exception';
            }
            $avisos[] = 'Existem vendas pendentes de aprovação/emissão ou sem incidência do imposto.';
        }
        return $avisos;
    }
    
    private function getReport($row){
        
        if($row['CDIMPOSTO'] == '00'){
            return array (
                array (
                    'name' => 'FII00800',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'Registro de Apuracao',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0
                    ),
                
                ),
                
                array (
                    'name' => 'FII00814',
                    'strategy' => 'QR2',
                    'extension' => '.xlsx',
                    'nmarquivo' => 'Documentos por Período - (Excel)',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0
                    ),
                ),
                
                array (
                    'name' => 'FII00300',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'Registro de Saidas',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0
                    ),
                ),
                
                array (
                    'name' => 'FII00100',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'Registro de Entradas',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0
                    ),
                ),
                
                array (
                    'name' => 'FIS96041',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'NFs Fora do Periodo',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0,
                        'IDENTRSAIDOP'    => 'T',
                        'CDFISCOPERIT'    => 'T',
                        'CDSITUTRIBIT'    => 'T',
                        'TCDMODDOCFISC'   => 'T',
                        'TCDTIPOOPER'     => 'T'
                    ),
                ),
                
                array (
                    'name' => 'FIS96055',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'Pre-Validacao SPED Fiscal',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'IDENTRSAIDOP'    => 'T'
                    ),
                ),
                
                array (
                    'name' => 'FIS96091',
                    'strategy' => 'QR2',
                    'extension' => '.pdf',
                    'nmarquivo' => 'Nota Fiscal Pendente',
                    'parameterMapping' => 
                    array (
                        'CDEMPRESA'       => $row['CDEMPRESA'],
                        'CDINSCESTA'      => $row['CDINSCESTA'],
                        'CDIMPOSTONFE'    => $row['CDIMPOSTO'],
                        'DTINICIO'        => $row['DTINIPERIODO'],
                        'DTFINAL'         => $row['DTFIMPERIODO'],
                        'NRPAGINA'        => isset($row['NRPAGINA']) ? $row['NRPAGINA'] : 0,
                        'IDENTRSAIDOP'    => 'T',
                        'CDFISCOPERIT'    => 'T',
                        'CDSITUTRIBIT'    => 'T',
                        'TCDMODDOCFISC'   => 'T',
                        'TCDTIPOOPER'     => 'T'
                    ),
                ),
                
            );
        }
        
    }
    
    public function salvarRelatorio($nomeArquivo, $reportUrl, $callServPath = null, $extensao) {
        $reportPath = null;
        try {

            $report = file_get_contents($reportUrl);
            
            $reportPath = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $nomeArquivo . $extensao;
                
            file_put_contents($reportPath, $report);
                
            if(isset($callServPath)){
                return $reportPath;
            } else {
                $filePath = $this->saveInFileUplouder($nomeArquivo, $reportPath, $extensao);
                unlink($reportPath);
                
                return $filePath;
            }
        
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveInFileUplouder($tmpFile , $fileName) {
        
        if (file_exists($tmpFile)) {
            $file = file_get_contents($tmpFile);
            $file = base64_encode($file);
        
            $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;
            
            $lastModify = filectime($tmpFile);
        
            $file = array(
                "name" => $fileName . $extensao, 
                "lastModified" => $lastModify,
                "lastModifiedDate" => '',
                "webkitRelativePath" => '',
                "type" => '',
                "size"  => filesize($tmpFile),
                "b64File" => $file);
                
            $row['FILE']    = array();
            $row['FILE'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'FILE', 'ApuracaoAutomatica');
            
            return $row['FILE'][0]['path'];
            
            /*$fileContent = file_get_contents($tmpFile);
            $fileBase64 = base64_encode($fileContent);
        
            $fileBase64Zeedhi = 'data: '.mime_content_type($tmpFile).';base64,'.$fileBase64;
            
            $row = [
                'FILE' => [
                    [
                        "name" => $fileName. $extensao, 
                        "type" => 'application/'.$extensao, 
                        "size" => strlen($fileContent), 
                        "path" => mime_content_type($tmpFile), 
                        "b64File" => $fileBase64Zeedhi
                    ]
                ],
                'FOLDER' => 'ApuracaoAutomatica'
            ];
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];*/
            
        } else {
            return false;
        }
    }
    
    public function deleteArquivos($row){
        try{
            
            if (strpos($row['NMPATHARQ'], '/tmp/') !== false) {
                if(isset($row['NMPATHARQ']) && file_exists($row['NMPATHARQ'])){
                    unlink($row['NMPATHARQ']);
                }
                $this->factoryApuracaoAutomatica->deleteArquivo($row);
            } else {
                $this->fileUploader->deleteFile($row['NMPATHARQ']);
            }
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
        
}
